# coding: utf-8

class Color:

    # 字体颜色
    black   = {"code": (30, 40), "en": "black",   "zh": "黑色"}
    red     = {"code": (31, 41), "en": "red",     "zh": "红色"}
    green   = {"code": (32, 42), "en": "green",   "zh": "绿色"}
    yellow  = {"code": (33, 43), "en": "yellow",  "zh": "黄色"}
    blue    = {"code": (34, 44), "en": "blue",    "zh": "蓝色"}
    magenta = {"code": (35, 45), "en": "magenta", "zh": "洋红"}
    cyan    = {"code": (36, 46), "en": "cyan",    "zh": "青色"}
    white   = {"code": (37, 47), "en": "white",   "zh": "白色"}


class Style:

    # 字体样式
    highlight  = {"code": (1,), "en": "highlight",  "zh": "高亮"}
    underline  = {"code": (4,), "en": "underline",  "zh": "下划线"}
    flash      = {"code": (5,), "en": "flash",      "zh": "闪烁"}
    reverse    = {"code": (7,), "en": "reverse",    "zh": "反显"}
    no_visible = {"code": (8,), "en": "no_visible", "zh": "不可见"}
    default    = {"code": ("", ""), "en": "default", "zh": "默认"}

class ColorPrintData(Color, Style):

    def __init__(self):
        self._all = []
        for name in self.__dir__():
            if not name.startswith("_") \
                and isinstance(getattr(self, name), dict):
                self._all.append(getattr(self, name))

    def set_color_or_style(self, data):
        # type(data) -> int
        if isinstance(data, int):
            data = self.__set_type_int(data)
        # type(data) -> str
        elif isinstance(data, str):
            data = self.__set_type_str(data)
        # type(data) -> list, tuple
        elif isinstance(data, (list, tuple)):
            data = self.__set_type_iter(data)
        # type(data) -> dict
        elif isinstance(data, dict):
            data = self.__set_type_dict(data)
        # other
        else:
            data = self.default
        return data

    def __set_type_int(self, data):
        for item in self._all:
            if item["code"] is not None and data in item["code"]:
                data = item
                break
        else:
            data = self.default
        return data

    def __set_type_str(self, data):
        if data.isdigit():
            return self.__set_type_int(int(data))
        data = data.lower()
        for item in self._all:
            if data == item["en"] or data == item["zh"]:
                data = item
                break
        else:
            data = self.default
        return data

    def __set_type_iter(self, data):
        return tuple(self.set_color_or_style(item) for item in data)

    def __set_type_dict(self, data):
        for item in self._all:
            if data == item:
                break
        else:
            data = self.default
        return data



from .main  import ColorPrint
from .config import Color, Style


__all__ = ["ColorPrint", "Color", "Style"]

# 实例
cp = ColorPrint()

# 彩色打印
cprint        = cp.cprint
print_black   = cp.print_black
print_red     = cp.print_red
print_green   = cp.print_green
print_yellow  = cp.print_yellow
print_blue    = cp.print_blue
print_magenta = cp.print_magenta
print_cyan    = cp.print_cyan
print_white   = cp.print_white

# 着色文本
color         = cp.colored
color_black   = cp.colored_black
color_red     = cp.colored_red
color_green   = cp.colored_green
color_yellow  = cp.colored_yellow
color_blue    = cp.colored_blue
color_magenta = cp.colored_magenta
color_cyan    = cp.colored_cyan
color_white   = cp.colored_white

# 进度条
progress_bar    = cp.progress_bar



import setuptools
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), "r", encoding="utf-8") as fp:
    long_description = fp.read()

setuptools.setup(

    # 包名称 - 注意: 使用 pypi 中存在包名会上传失败
    name="cprints",
    # 包版本号，便于维护版本
    version="1.0.7",
    # 自己项目地址，比如github的项目地址
    url="https://github.com/PengKe-x/cprins",
    # 作者，可以写自己的姓名
    author="PengKe",
    # 作者联系方式，可写自己的邮箱地址
    author_email="925330867@qq.com",
    # 包的简述
    description="输出彩色文字",
    # 包的详细介绍，一般在 README.md 文件内
    long_description=long_description,
    # 文件 README.md 阅读支持类型
    long_description_content_type="text/markdown",

    # 表明当前模块依赖哪些包，若环境中没有，则会从 pypi 中下载安装
    install_requires=[],
    # setup.py 本身要依赖的包，这通常是为一些setuptools的插件准备的配置
    # 这里列出的包，不会自动安装。
    setup_requires=[],
    # 用于安装setup_requires或tests_require里的软件包
    # 这些信息会写入egg的 metadata 信息中
    dependency_links=[],
     # install_requires 在安装模块时会自动安装依赖包
    # 而 extras_require 不会，这里仅表示该模块会依赖这些包
    # 但是这些包通常不会使用到，只有当你深度使用模块时，才会用到，这里需要你手动安装
    # extras_require= {}

    # 你要安装的包，通过 setuptools.find_packages 找到当前目录下有哪些包
    packages=setuptools.find_packages(),

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # 对 Python 的最低版本要求
    python_requires='>=3.6',
)


import math
from .config import ColorPrintData

PREFIX = "\033["
SUFFIX = "\033[0m"

class ColorPrint(ColorPrintData):

    def color(self, text, /, *args, fg=None, bg=None, style=None, suffix=True):
        """
        :params text   : 着色文本
        :params args   : print可变位置参数
        :params fg     : 前景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params bg     : 背景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params style  : 文本样式(default, highlight, underline, flash, reverse)
        :params suffix : 恢复默认打印方式

        explain:

        from cprints import cp
        text = cp.color("Hello World", fg=cp.red, bg=cp.black, style=cp.underline)
        print(text)
        """
        fg = self.set_color_or_style(fg)
        bg = self.set_color_or_style(bg)
        style  = self.set_color_or_style(style)
        # 字体颜色
        if isinstance(fg, tuple):
            fg = fg[0]
        fg = str(fg["code"][0])
        # 背景颜色
        if isinstance(bg, tuple):
            bg = bg[0]
        bg = str(bg["code"][1])
        # 字体样式
        if not isinstance(style, tuple):
            style = (style, )
        style = ";".join(set([str(item["code"][0]) for item in style]))
        # 拼接
        data  = ";".join([item for item in (style, fg, bg) if item])
        if not data:
            data = str(0)
        suffix = SUFFIX if bool(suffix) else ""
        args   = " ".join([str(i) for i in args])
        if args:
            args = " " + args
        return f"{PREFIX}{data}m{text}{args}{suffix}"

    colored = color


    def cprint(self, text, /, *args, fg=None, bg=None, style=None, suffix=True, **kwargs):
        """
        :params text   : 着色文本
        :params args   : print可变位置参数
        :params fg     : 前景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params bg     : 背景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params style  : 文本样式(default, highlight, underline, flash, reverse)
        :params suffix : 恢复默认打印方式
        :params kwargs : print内置函数可变关键字参数

        explain:

        from cprints import cp
        cp.cprint("Hello World", fg=cp.red, bg=cp.black, style=cp.underline)
        """
        text = self.color(text, *args, fg=fg, bg=bg, style=style, suffix=suffix)
        print(text, **kwargs)


    def progress_bar(self, now: int, total: int, lenght=36, bar=b"\xe2\x94\x81", pad=""):
        """
        :params now    : 当前进度值
        :params total  : 总进度值
        :params lenght : 进度条长度
        :params bar    : 进度条 进度填充
        :params pad    : 进度条 其余填充

        explain:

        from cprints import cp
        maximum = 100
        for i in range(1, maximum+1):
            cp.progress_bar(now=i, total=maximum)
        """
        now = total if now >= total else now
        bar = bar.decode() if isinstance(bar, bytes) else str(bar)
        # 百分比
        percent = now / total
        # 进度条总长度
        total_bar  = len(self.color(lenght * bar))
        foreground = self.red if now < total else self.green
        # 进度条
        progress_bar = self.color(
            math.floor(percent * lenght) * bar, fg=foreground)
        # 需要打印的百分比
        percent = self.color(f"{percent:.2%}", fg=self.magenta)
        _num    = self.color(f"{now}/{total}", fg=self.yellow)
        end     = "\n" if now >= total else ""
        print(f"\r{percent: >16} {progress_bar:{pad}<{total_bar}} {_num} ", end=end, flush=True)


    def color_black(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.black, bg=bg, style=style, suffix=suffix)

    colored_black = color_black

    def color_red(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.red, bg=bg, style=style, suffix=suffix)

    colored_red = color_red

    def color_green(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.green, bg=bg, style=style, suffix=suffix)

    colored_green = color_green

    def color_yellow(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.yellow, bg=bg, style=style, suffix=suffix)

    colored_yellow = color_yellow

    def color_blue(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.blue, bg=bg, style=style, suffix=suffix)

    colored_blue = color_blue

    def color_magenta(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.magenta, bg=bg, style=style, suffix=suffix)

    colored_magenta = color_magenta

    def color_cyan(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.cyan, bg=bg, style=style, suffix=suffix)

    colored_cyan = color_cyan

    def color_white(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.color(text, *args, fg=self.white, bg=bg, style=style, suffix=suffix)

    colored_white = color_white

    def print_black(self, text ,/, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.black, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_red(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.red, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_green(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.green, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_yellow(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.yellow, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_blue(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.blue, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_magenta(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.magenta, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_cyan(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.cyan, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_white(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.white, bg=bg, style=style, suffix=suffix, **kwargs)
